# Selectricity: Voting Machinery for the Masses
# Copyright (C) 2007, 2008 Benjamin Mako Hill <mako@atdot.cc>
# Copyright (C) 2007 Massachusetts Institute of Technology
#
# This program is free software. Please see the COPYING file for
# details.

class ElectionController < ApplicationController
  require_dependency "raw_voter_list"
  require_dependency "voter"
  require_dependency "vote"
  require_dependency "candidate"

  helper :sparklines
  layout 'main'


  before_filter :verify_owner,
    :except => [:new, :general_information, :create_election]
  before_filter :verify_not_active,
    :except => [:new, :general_information, :create_election,
                :show, :results, :details, :pref_tables]

  ## methods for displaying, creating,
  ## and manipulating election overview data
  ####################################################################

  def new
    redirect_to :action => 'general_information'
  end
  
  def general_information
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'overview' }
    @election = Election.new
    render :action => 'general_information'
  end
  
  def create_election
    @election = Election.new(params[:election])
    
    # default options
    @election.user = session[:user]
    @election.anonymous = 1
    @election.startdate = Time.now
    @election.type = 'Election'
    
    holder = create_theme_hash
    unless holder.values.all? {|v| v.has_value?("")}
      token_generator = UniqueTokenGenerator.new( 16 )
      @election.embed_custom_string = token_generator.token
      add_theme(@election.embed_custom_string)
    end
    
    if @election.save
      flash[:notice] = 'Election was successfully created.'
      redirect_to :action => 'edit_candidates', :id => @election.id
    else
      render :action => 'general_information'
    end
  end
  
  def edit_general_information
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'overview' }
    @election = Election.find(params[:id])
  end
  
  def update_general_information
    @election = Election.find(params[:id])
    
    holder = create_theme_hash
    unless holder.values.all? {|v| v.has_value?("")}
      unless @election.embed_custom_string
        token_generator = UniqueTokenGenerator.new( 16 )
        @election.embed_custom_string = token_generator.token
      end
      
      add_theme(@election.embed_custom_string)
    end
    
    if @election.update_attributes(params[:election])
      flash[:notice] = 'Election was successfully updated.'
      redirect_to :action => 'show', :id => @election
    else
      render :action => 'edit_general_information'
    end
  end
  
  #Takes care of uploading custom images 
  #unnecessarily long, how can I compress?
  def add_theme(prefix)
    holder = create_theme_hash
    unless params[:top_bar][:uploaded_data].to_s.empty?
      previous = SkinPicture.find(:first,
      :conditions => ["filename = ?", @election.embed_custom_string + "top_bar.png"])
      if previous
        previous.destroy
      end
      top_bar = SkinPicture.new(params[:top_bar])
      top_bar.filename = prefix + "top_bar." + params[:top_bar][:uploaded_data].content_type[6..-2]
      top_bar.save
    end
    unless params[:default_image][:uploaded_data].to_s.empty?
      previous = SkinPicture.find(:first,
      :conditions => ["filename = ?", @election.embed_custom_string + "default_image.png"])
      if previous
        previous.destroy
      end
      default_image = SkinPicture.new(params[:default_image])
      default_image.filename = prefix + "default_image." + params[:default_image][:uploaded_data].content_type[6..-2]
      default_image.save
    end
    unless params[:bg1][:uploaded_data].to_s.empty?
      previous = SkinPicture.find(:first,
      :conditions => ["filename = ?", @election.embed_custom_string + "bg1.png"])
      if previous
        previous.destroy
      end
      bg1 = SkinPicture.new(params[:bg1])  
      bg1.filename = prefix + "bg1." + params[:bg1][:uploaded_data].content_type[6..-2]
      bg1.save
    end
    unless params[:bg2][:uploaded_data].to_s.empty?
      previous = SkinPicture.find(:first,
      :conditions => ["filename = ?", @election.embed_custom_string + "bg2.png"])
      if previous
        previous.destroy
      end
      bg2 = SkinPicture.new(params[:bg2]) 
      bg2.filename = prefix + "bg2." + params[:bg2][:uploaded_data].content_type[6..-2]
      bg2.save
    end
    unless params[:bottom_bar][:uploaded_data].to_s.empty?
      previous = SkinPicture.find(:first,
      :conditions => ["filename = ?", @election.embed_custom_string + "bottom_bar.png"])
      if previous
        previous.destroy
      end
      bottom_bar = SkinPicture.new(params[:bottom_bar])
      bottom_bar.filename = prefix + "bottom_bar." + params[:bottom_bar][:uploaded_data].content_type[6..-2]
      bottom_bar.save
    end
        
  end
  
  def show
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'review' }

    @election = Election.find(params[:id])
    if @election.class  == QuickVote
      redirect_to(:controller => 'quickvote', :action => 'index', :ident => @election.id)
    end
      
  end

  def start_election
    @election = Election.find(params[:id])
    
    @election.voters.each do |voter|
      voter.vote = Vote.new
      email_voter voter unless voter.email.nil?
    end

    @election.activate!
    redirect_to :action => 'show', :id => @election.id
  end

  # methods fod display, adding, deleting, and manipulating candidate
  # information for elections
  ####################################################################
  def edit_candidates
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'candidates' }
    @election = Election.find(params[:id] )
  end

  def add_candidate
    @election = Election.find(params[:id])
    @candidate = Candidate.new(params[:candidate])
    @election.candidates << @candidate

    if @candidate.save
      # check to see if they've uploaded a picture
      if params[:picture][:uploaded_data]
        picture = Picture.new(params[:picture])
        @candidate.picture = picture if picture.save
      end

      @candidate = Candidate.new
      redirect_to :action => 'edit_candidates', :id => @election.id
    else
      render :action => 'edit_candidates', :id => @election.id
    end
  end
  
  def delete_candidate
    candidate = Candidate.find(params[:candidate] )
    candidate.destroy
  end

  def candidate_picture
    candidate = Candidate.find(params[:candidate])
    send_data( candidate.picture.data,
               :filename => candidate.picture.filename,
	       :type => candidate.picture.filetype,
	       :disposition => 'inline' )
  end

  ## methods for displaying, adding, deleting, and manipulating voters
  ## for a particular election
  ####################################################################
  def new_voters
    redirect_to :action => 'edit_voters', :id => params[:id]
  end
  
  def edit_voters
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'voters' }

    @election = Election.find(params[:id])

    if params.has_key?( :raw_voter_list )
      process_incoming_voters( params[:raw_voter_list] )
    end
    @edit = true
    @raw_voter_list = RawVoterList.new
  end
  
  def delete_voter
    voter = FullVoter.find(params[:voter])
    voter.destroy
  end

  def toggle_authenticated
    @election = Election.find(params[:id])
    if params[:authenticated] == "1"
      @election.authenticated = true
    else
      @election.authenticated = false
    end
    @election.save
  end
  
  ## methods for computing and printing results
  ####################################################################
  def results
    @election = Election.find(params[:id])

    if @election.early_results? \
       or @election.enddate < Time.now
      
      # render results
      @sidebar_content = render_to_string(:partial => 'full_results_sidebar')
      render :template => 'common/results'
    else
      redirect_to :action => 'index'
    end
  end
  
  def pref_tables
    @election = Election.find(params[:id])
    render :template => 'common/pref_tables_wrapper', :layout => 'basic'
  end

  def details
    @election = Election.find(params[:id])
    render :template => 'common/details'
  end

  ## private methods
  ####################################################################
  private

    def process_incoming_voters(raw_voter_list)
      incoming_voters = RawVoterList.new( raw_voter_list )

      unless incoming_voters.entries.empty?
        incoming_voters.each do |new_voter|
          new_voter.email.strip! # There's a trailing \r on all but the last in
                                 # the list!
          if new_voter.valid?
            # the new voter should be in good shape. save add to the election
            @election.voters << new_voter
	          new_voter.save
          end
          # TODO: Can we do some kind of AJAX error message for the voter being invalid?
        end
        @election.save
      end
 
      # reset the next time to have a the same default value for emailing
      @raw_voter_list = RawVoterList.new
      @raw_voter_list.email = incoming_voters.email
    end

    def email_voter(voter=nil)
      if voter
        VoterNotify.deliver_votestart(voter)
        voter.contacted=1
        voter.save
      end
    end
  
    def create_theme_hash
      target = Hash.new
      params.each do |k,v|
        target[k] = v if k=="top_bar" or k=="default_image" or k=="bg1" \
                      or k=="bg2" or k=="bottom_bar"
      end
      return target
    end

    # verify that the person trying to edit the election is the owner
    def verify_owner
      election = Election.find(params[:id])
      unless election.user == session[:user]
        redirect_to :controller => 'front', :action => 'index' 
      end
    end

    # verify that the election is not active
    def verify_not_active
      election = Election.find(params[:id])
      unless election.active == 0
        redirect_to :controller => 'front', :action => 'index' 
      end
    end

end
